import os
from django.conf import settings


def ensure_dirs():
    path = os.path.join(settings.MEDIA_ROOT, 's3', 'uploads')
    if not os.path.exists(path):
        os.makedirs(path)

    path = os.path.join(settings.MEDIA_ROOT, 's3', 'downloads')
    if not os.path.exists(path):
        os.makedirs(path)



import string
import random
from django.conf import settings


def validate_settings():
    assert settings.AWS, \
        "No AWS settings found"
    assert settings.AWS.get('ACCESS_KEY'), \
        "AWS access key is not set in settings"
    assert settings.AWS.get('SECRET_KEY'), \
        "AWS secret key is not set in settings"
    assert settings.AWS.get('BUCKET'), \
        "AWS bucket name is not set in settings"


ID_FIELD_LENGTH = 24

alphabet = string.ascii_lowercase + string.digits
for loser in 'l1o0':
    i = alphabet.index(loser)
    alphabet = alphabet[:i] + alphabet[i + 1:]


def byte_to_base32_chr(byte):
    return alphabet[byte & 31]


def random_id():
    rand_id = [random.randint(0, 0xFF) for i in range(ID_FIELD_LENGTH)]
    return ''.join(map(byte_to_base32_chr, rand_id))



from django.views.decorators.http import require_POST
from django.http import JsonResponse
from django.utils.translation import ugettext as _
from django.core.files.storage import default_storage

from buckets.exceptions import InvalidPayload


def validate_payload(payload):
    errors = {}

    key = payload.get('key', None)
    if not key:
        errors['key'] = _("'key' is required")

    if errors:
        raise InvalidPayload(errors=errors)


@require_POST
def signed_url(request):
    if not hasattr(default_storage, 'get_signed_url'):
        response = {'error': 'Not found'}
        status = 404
    else:
        try:
            validate_payload(request.POST)
            response = default_storage.get_signed_url(key=request.POST['key'])
            status = 200
        except InvalidPayload as e:
            response = e.errors
            status = 400

    return JsonResponse(response, status=status)



import os
import urllib

from django.conf import settings


class FakeS3Storage(object):
    def __init__(self, dir=None):
        self.dir = dir or os.path.join(settings.MEDIA_ROOT, 's3')

    def open(self, url):
        name = os.path.basename(urllib.request.url2pathname(url))
        uploaded = os.path.join(self.dir, 'uploads', name)

        with open(os.path.join(self.dir, 'downloads', name), 'wb') as dest:
            dest.write(open(uploaded, 'rb').read())
            dest.close()

        return dest.name

    def save(self, name, content):
        url = '/media/s3/uploads/' + name

        with open(os.path.join(self.dir, 'uploads', name), 'wb') as dest:
            dest.write(content)

        return url

    def delete(self, url):
        name = os.path.basename(urllib.request.url2pathname(url))
        uploaded = os.path.join(self.dir, 'uploads', name)
        os.remove(uploaded)

    def get_signed_url(self, key=None):
        return {'url': '/media/s3/uploads', 'fields': {'key': key}}



import pytest
import shutil
import os

from django.conf import settings

from .utils import ensure_dirs


@pytest.fixture(scope='function')
def make_dirs(request):
    ensure_dirs()

    def teardown():
        shutil.rmtree(os.path.join(settings.MEDIA_ROOT, 's3'))
    request.addfinalizer(teardown)


def create_file():
    path = os.path.join(settings.MEDIA_ROOT, 's3', 'text.txt')
    file = open(path, 'wb')
    file.write('Some content'.encode('utf-8'))
    file.close()

    return file



from os.path import basename
from django.forms import widgets
from django.utils.safestring import mark_safe


class S3FileUploadWidget(widgets.TextInput):
    default_html = (
        '<div class="s3-buckets {uploaded_class}"'
        '     data-upload-to="{upload_to}">'
        '   <div class="file-links">'
        '       <a class="file-link" href="{file_url}">{file_name}</a>'
        '       <a class="file-remove" href="#">(Remove)</a>'
        '   </div>'
        '   <input class="file-url" type="hidden" value="{file_url}"'
        '          id="{element_id}" name="{name}" />'
        '   <input class="file-input" type="file" />'
        '</div>'
    )

    class Media:
        js = (
            'buckets/js/script.js',
        )
        css = {
            'all': (
                'buckets/css/buckets.css',
            )
        }

    def __init__(self, upload_to='', *args, **kwargs):
        self.html = kwargs.pop('html', self.default_html)
        self.upload_to = upload_to

        super(S3FileUploadWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if isinstance(value, str):
            file_url = value
        else:
            file_url = value.url if value else ''

        output = self.html.format(
            name=name,
            file_url=file_url,
            element_id=self.build_attrs(attrs).get('id'),
            file_name=basename(file_url) if file_url else '',
            uploaded_class=('uploaded' if value else ''),
            upload_to=self.upload_to
        )

        return mark_safe(output)



import os
from django.utils.translation import ugettext as _
from django.core.files.storage import default_storage
from django.db import models

from .widgets import S3FileUploadWidget


class S3File(object):
    """ This is the internal value an `S3FileField`. It provides access to the
        actual file on S3, e.g. for post-processing. It usually uses an
        instance of S3Storage to download, upload or delete the file on S3."""
    def __init__(self, url, field):
        self.field = field
        self.storage = field.storage
        self.url = url
        self.committed = True

    def _get_file(self):
        if not hasattr(self, '_file') or not self._file:
            path = self.storage.open(self.url)
            self._file = open(path, 'rb')
        return self._file

    def _set_file(self, file):
        self._file = file
        self.committed = False

    def _del_file(self):
        self.storage.delete(self.url)
        if hasattr(self, '_file'):
            del self._file

        self.url = ''

    file = property(_get_file, _set_file, _del_file)

    def open(self):
        self._get_file()
        return self._file

    def save(self):
        if not self.committed:
            name = os.path.join(self.field.upload_to,
                                os.path.basename(self._file.name))
            self.url = self.storage.save(name, self._file.read())
            self.committed = True

        return self.url

    def delete(self):
        self._del_file()


class S3FileDescriptor(object):
    def __init__(self, field):
        self.field = field

    def __get__(self, instance, value):
        return instance.__dict__[self.field.name]

    def __set__(self, instance, value):
        if isinstance(value, S3File):
            instance.__dict__[self.field.name] = value
        else:
            instance.__dict__[self.field.name] = S3File(value, self.field)


class S3FileField(models.Field):
    """ Stores and provides access to a file stored in an AWS S3 bucket. To set
        the value of an `S3FileField` you usually provide the URL of the file
        in the bucket. Internally this URL is converted to a `S3File`, which
        provides access to the actual file on S3."""
    attr_class = S3File
    descriptor_class = S3FileDescriptor

    description = _("A file stored in an AWS S3 buckets.")

    def __init__(self, upload_to='', storage=None, *args, **kwargs):
        self.storage = storage or default_storage
        self.upload_to = upload_to
        self.widget = S3FileUploadWidget(upload_to=upload_to)

        kwargs['max_length'] = kwargs.get('max_length', 200)
        super(S3FileField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return 'CharField'

    def contribute_to_class(self, cls, name, **kwargs):
        super(S3FileField, self).contribute_to_class(cls, name, **kwargs)
        setattr(cls, self.name, self.descriptor_class(self))

    def deconstruct(self):
        name, path, args, kwargs = super(S3FileField, self).deconstruct()

        if self.upload_to != '':
            kwargs['upload_to'] = self.upload_to

        if self.storage != default_storage:
            kwargs['storage'] = self.storage

        if kwargs.get('max_length') == 200:
            del kwargs['max_length']

        return name, path, args, kwargs

    def from_db_value(self, value, expression, connection, context):
        return S3File(value, self)

    def to_python(self, value):
        if value is None or isinstance(value, S3File):
            return value

        return S3File(value, self)

    def get_prep_value(self, value):
        if isinstance(value, S3File):
            return value.url

        return value

    def pre_save(self, model_instance, add):
        file = getattr(model_instance, self.name)
        file.save()

        return file.url

    def formfield(self, *args, **kwargs):
        kwargs['widget'] = self.widget
        return super(S3FileField, self).formfield(*args, **kwargs)



from django.conf.urls import include, url
from buckets.test import views

urlpatterns = [
    url(r'^', include('buckets.urls')),
    url(r'^media/s3/uploads$', views.fake_s3_upload, name='fake_s3_upload'),
]
